// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

mod buffer;
pub use self::buffer::Buffer;

mod completion;
pub use self::completion::Completion;

mod completion_cell;
pub use self::completion_cell::CompletionCell;

mod completion_context;
pub use self::completion_context::CompletionContext;

mod completion_proposal;
pub use self::completion_proposal::CompletionProposal;

mod completion_provider;
pub use self::completion_provider::CompletionProvider;

mod completion_snippets;
pub use self::completion_snippets::CompletionSnippets;

mod completion_words;
pub use self::completion_words::CompletionWords;

mod file;
pub use self::file::File;

mod file_loader;
pub use self::file_loader::FileLoader;

mod file_saver;
pub use self::file_saver::FileSaver;

mod gutter;
pub use self::gutter::Gutter;

mod gutter_lines;
pub use self::gutter_lines::GutterLines;

mod gutter_renderer;
pub use self::gutter_renderer::GutterRenderer;

mod gutter_renderer_pixbuf;
pub use self::gutter_renderer_pixbuf::GutterRendererPixbuf;

mod gutter_renderer_text;
pub use self::gutter_renderer_text::GutterRendererText;

mod hover;
pub use self::hover::Hover;

mod hover_context;
pub use self::hover_context::HoverContext;

mod hover_display;
pub use self::hover_display::HoverDisplay;

mod hover_provider;
pub use self::hover_provider::HoverProvider;

mod indenter;
pub use self::indenter::Indenter;

mod language;
pub use self::language::Language;

mod language_manager;
pub use self::language_manager::LanguageManager;

mod map;
pub use self::map::Map;

mod mark;
pub use self::mark::Mark;

mod mark_attributes;
pub use self::mark_attributes::MarkAttributes;

mod print_compositor;
pub use self::print_compositor::PrintCompositor;

mod region;
pub use self::region::Region;

mod search_context;
pub use self::search_context::SearchContext;

mod search_settings;
pub use self::search_settings::SearchSettings;

mod snippet;
pub use self::snippet::Snippet;

mod snippet_chunk;
pub use self::snippet_chunk::SnippetChunk;

mod snippet_context;
pub use self::snippet_context::SnippetContext;

mod snippet_manager;
pub use self::snippet_manager::SnippetManager;

mod space_drawer;
pub use self::space_drawer::SpaceDrawer;

mod style;
pub use self::style::Style;

mod style_scheme;
pub use self::style_scheme::StyleScheme;

mod style_scheme_chooser;
pub use self::style_scheme_chooser::StyleSchemeChooser;

mod style_scheme_chooser_button;
pub use self::style_scheme_chooser_button::StyleSchemeChooserButton;

mod style_scheme_chooser_widget;
pub use self::style_scheme_chooser_widget::StyleSchemeChooserWidget;

mod style_scheme_manager;
pub use self::style_scheme_manager::StyleSchemeManager;

#[cfg(feature = "v5_4")]
#[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
mod style_scheme_preview;
#[cfg(feature = "v5_4")]
#[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
pub use self::style_scheme_preview::StyleSchemePreview;

mod tag;
pub use self::tag::Tag;

mod view;
pub use self::view::View;

#[cfg(feature = "v5_4")]
#[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
mod vim_im_context;
#[cfg(feature = "v5_4")]
#[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
pub use self::vim_im_context::VimIMContext;

mod encoding;
pub use self::encoding::Encoding;

mod enums;
pub use self::enums::BackgroundPatternType;
pub use self::enums::BracketMatchType;
pub use self::enums::ChangeCaseType;
pub use self::enums::CompletionActivation;
pub use self::enums::CompletionColumn;
pub use self::enums::CompressionType;
pub use self::enums::FileLoaderError;
pub use self::enums::FileSaverError;
pub use self::enums::GutterRendererAlignmentMode;
pub use self::enums::NewlineType;
pub use self::enums::SmartHomeEndType;
pub use self::enums::ViewGutterPosition;

mod flags;
pub use self::flags::FileSaverFlags;
pub use self::flags::SortFlags;
pub use self::flags::SpaceLocationFlags;
pub use self::flags::SpaceTypeFlags;

pub(crate) mod functions;

pub(crate) mod traits {
    pub use super::buffer::BufferExt;
    pub use super::completion_proposal::CompletionProposalExt;
    pub use super::completion_provider::CompletionProviderExt;
    pub use super::completion_snippets::CompletionSnippetsExt;
    pub use super::completion_words::CompletionWordsExt;
    pub use super::file::SourceFileExt;
    pub use super::gutter_renderer::GutterRendererExt;
    pub use super::gutter_renderer_pixbuf::GutterRendererPixbufExt;
    pub use super::gutter_renderer_text::GutterRendererTextExt;
    pub use super::hover_provider::HoverProviderExt;
    pub use super::indenter::IndenterExt;
    pub use super::map::MapExt;
    pub use super::mark::MarkExt;
    pub use super::print_compositor::PrintCompositorExt;
    pub use super::region::RegionExt;
    pub use super::search_settings::SearchSettingsExt;
    pub use super::style_scheme_chooser::StyleSchemeChooserExt;
    pub use super::tag::TagExt;
    pub use super::view::ViewExt;
}
pub(crate) mod builders {
    pub use super::buffer::BufferBuilder;
    pub use super::completion::CompletionBuilder;
    pub use super::completion_cell::CompletionCellBuilder;
    pub use super::completion_context::CompletionContextBuilder;
    pub use super::completion_snippets::CompletionSnippetsBuilder;
    pub use super::completion_words::CompletionWordsBuilder;
    pub use super::file::FileBuilder;
    pub use super::file_loader::FileLoaderBuilder;
    pub use super::file_saver::FileSaverBuilder;
    pub use super::gutter::GutterBuilder;
    pub use super::gutter_renderer_pixbuf::GutterRendererPixbufBuilder;
    pub use super::gutter_renderer_text::GutterRendererTextBuilder;
    pub use super::hover::HoverBuilder;
    pub use super::hover_display::HoverDisplayBuilder;
    pub use super::language_manager::LanguageManagerBuilder;
    pub use super::map::MapBuilder;
    pub use super::mark::MarkBuilder;
    pub use super::mark_attributes::MarkAttributesBuilder;
    pub use super::print_compositor::PrintCompositorBuilder;
    pub use super::region::RegionBuilder;
    pub use super::search_context::SearchContextBuilder;
    pub use super::search_settings::SearchSettingsBuilder;
    pub use super::snippet::SnippetBuilder;
    pub use super::snippet_chunk::SnippetChunkBuilder;
    pub use super::snippet_manager::SnippetManagerBuilder;
    pub use super::space_drawer::SpaceDrawerBuilder;
    pub use super::style::StyleBuilder;
    pub use super::style_scheme::StyleSchemeBuilder;
    pub use super::style_scheme_chooser_button::StyleSchemeChooserButtonBuilder;
    pub use super::style_scheme_chooser_widget::StyleSchemeChooserWidgetBuilder;
    pub use super::style_scheme_manager::StyleSchemeManagerBuilder;
    #[cfg(feature = "v5_4")]
    #[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
    pub use super::style_scheme_preview::StyleSchemePreviewBuilder;
    pub use super::tag::TagBuilder;
    pub use super::view::ViewBuilder;
    #[cfg(feature = "v5_4")]
    #[cfg_attr(docsrs, doc(cfg(feature = "v5_4")))]
    pub use super::vim_im_context::VimIMContextBuilder;
}
